"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDriveId = exports.getChildrenByDriveItem = exports.getDriveItemFromDriveAndItem = exports.getDriveItemByServerRelativePath = exports.getDriveItemByRootFileName = void 0;
const odspDocLibUtils_js_1 = require("./odspDocLibUtils.js");
const odspErrorUtils_js_1 = require("./odspErrorUtils.js");
const odspRequest_js_1 = require("./odspRequest.js");
/**
 * @internal
 */
async function getDriveItemByRootFileName(server, account, path, authRequestInfo, create, driveId) {
    const accountPath = account !== undefined ? `/${account}` : "";
    let getDriveItemUrl;
    if (driveId !== undefined && driveId !== "") {
        const encodedDrive = encodeURIComponent(driveId);
        getDriveItemUrl = `${(0, odspDocLibUtils_js_1.getSiteUrl)(server)}${accountPath}/_api/v2.1/drives/${encodedDrive}/root:${path}:`;
    }
    else {
        getDriveItemUrl = `${(0, odspDocLibUtils_js_1.getSiteUrl)(server)}${accountPath}/_api/v2.1/drive/root:${path}:`;
    }
    return getDriveItem(getDriveItemUrl, authRequestInfo, create);
}
exports.getDriveItemByRootFileName = getDriveItemByRootFileName;
/**
 * @internal
 */
async function getDriveItemByServerRelativePath(server, serverRelativePath, authRequestInfo, create) {
    let account = "";
    const pathParts = serverRelativePath.split("/");
    if (serverRelativePath.startsWith("/")) {
        pathParts.shift();
    }
    if (pathParts.length === 0) {
        throw new Error(`Invalid serverRelativePath ${serverRelativePath}`);
    }
    if (pathParts.length >= 2 &&
        (pathParts[0] === "personal" || pathParts[0] === "teams" || pathParts[0] === "sites")) {
        account = `${pathParts.shift()}/${pathParts.shift()}`;
    }
    const library = pathParts.shift();
    if (!library) {
        // Default drive/library
        return getDriveItemByRootFileName(server, account, "/", authRequestInfo, create);
    }
    const path = `/${pathParts.join("/")}`;
    const driveId = await getDriveId(server, account, library, authRequestInfo);
    const getDriveItemUrl = `${(0, odspDocLibUtils_js_1.getSiteUrl)(server)}/_api/v2.1/drives/${driveId}/root:${path}:`;
    return getDriveItem(getDriveItemUrl, authRequestInfo, create);
}
exports.getDriveItemByServerRelativePath = getDriveItemByServerRelativePath;
/**
 * @internal
 */
async function getDriveItemFromDriveAndItem(server, drive, item, authRequestInfo) {
    const url = `${(0, odspDocLibUtils_js_1.getSiteUrl)(server)}/_api/v2.1/drives/${drive}/items/${item}`;
    return getDriveItem(url, authRequestInfo, false);
}
exports.getDriveItemFromDriveAndItem = getDriveItemFromDriveAndItem;
/**
 * @internal
 */
async function getChildrenByDriveItem(driveItem, server, authRequestInfo) {
    if (!driveItem.isFolder) {
        return [];
    }
    let url = `${(0, odspDocLibUtils_js_1.getSiteUrl)(server)}/_api/v2.1/drives/${driveItem.driveId}/items/${driveItem.itemId}/children`;
    let children = [];
    do {
        const response = await (0, odspRequest_js_1.getAsync)(url, authRequestInfo);
        if (response.status !== 200) {
            // pre-0.58 error message: unableToGetChildren
            (0, odspErrorUtils_js_1.throwOdspNetworkError)("Unable to get driveItem children", response.status, response);
        }
        const getChildrenResult = await response.json();
        children = children.concat(getChildrenResult.value);
        url = getChildrenResult["@odata.nextLink"];
    } while (url);
    return children.map(toIODSPDriveItem);
}
exports.getChildrenByDriveItem = getChildrenByDriveItem;
async function getDriveItem(getDriveItemUrl, authRequestInfo, create) {
    let response = await (0, odspRequest_js_1.getAsync)(getDriveItemUrl, authRequestInfo);
    if (response.status !== 200) {
        if (!create) {
            // pre-0.58 error message: unableToGetDriveItemIdFromPath
            (0, odspErrorUtils_js_1.throwOdspNetworkError)("Unable to get drive/item id from path", response.status, response);
        }
        // Try creating the file
        const contentUri = `${getDriveItemUrl}/content`;
        const createResultResponse = await (0, odspRequest_js_1.putAsync)(contentUri, authRequestInfo);
        if (createResultResponse.status !== 201) {
            // pre-0.58 error message: failedToCreateFile
            (0, odspErrorUtils_js_1.throwOdspNetworkError)("Failed to create file", createResultResponse.status, createResultResponse);
        }
        response = await (0, odspRequest_js_1.getAsync)(getDriveItemUrl, authRequestInfo);
        if (response.status !== 200) {
            // pre-0.58 error message: unableToGetDriveItemIdFromPath
            (0, odspErrorUtils_js_1.throwOdspNetworkError)("Unable to get drive/item id from path after creating", response.status, response);
        }
    }
    const getDriveItemResult = await response.json();
    return toIODSPDriveItem(getDriveItemResult);
}
/**
 * @legacy
 * @alpha
 */
async function getDriveId(server, account, library, authRequestInfo) {
    if (library === undefined) {
        const drive = await getDefaultDrive(server, account, authRequestInfo);
        return drive.id;
    }
    const drives = await getDrives(server, account, authRequestInfo);
    const accountPath = account ? `/${account}` : "";
    const drivePath = encodeURI(`${(0, odspDocLibUtils_js_1.getSiteUrl)(server)}${accountPath}/${library}`);
    const index = drives.findIndex((value) => value.webUrl === drivePath);
    if (index === -1) {
        throw Error(`Drive ${drivePath} not found.`);
    }
    return drives[index].id;
}
exports.getDriveId = getDriveId;
async function getDefaultDrive(server, account, authRequestInfo) {
    const response = await getDriveResponse("drive", server, account, authRequestInfo);
    const getDriveResult = await response.json();
    return getDriveResult;
}
async function getDrives(server, account, authRequestInfo) {
    const response = await getDriveResponse("drives", server, account, authRequestInfo);
    const getDriveResult = await response.json();
    return getDriveResult.value;
}
async function getDriveResponse(routeTail, server, account, authRequestInfo) {
    const accountPath = account ? `/${account}` : "";
    const getDriveUrl = `${(0, odspDocLibUtils_js_1.getSiteUrl)(server)}${accountPath}/_api/v2.1/${routeTail}`;
    const response = await (0, odspRequest_js_1.getAsync)(getDriveUrl, authRequestInfo);
    if (response.status !== 200) {
        (0, odspErrorUtils_js_1.throwOdspNetworkError)(
        // pre-0.58 error message: failedToGetDriveResponse
        `Failed to get response from /${routeTail} endpoint`, response.status, response, undefined, { route: routeTail });
    }
    return response;
}
function toIODSPDriveItem(parsedDriveItemBody) {
    const path = parsedDriveItemBody.parentReference.path
        ? parsedDriveItemBody.parentReference.path.split("root:")[1]
        : "/";
    return {
        path,
        name: parsedDriveItemBody.name,
        driveId: parsedDriveItemBody.parentReference.driveId,
        itemId: parsedDriveItemBody.id,
        isFolder: !!parsedDriveItemBody.folder,
    };
}
//# sourceMappingURL=odspDrives.js.map